package org.terracotta.plugins.lucene_2_0_0;

import java.io.File;
import java.io.IOException;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;

import com.tc.object.config.ConfigLockLevel;
import com.tc.object.config.ConfigVisitor;
import com.tc.object.config.DSOClientConfigHelper;
import com.tc.object.config.LockDefinition;
import com.tc.simulator.app.ApplicationConfig;
import com.tc.simulator.listener.ListenerProvider;
import com.tc.test.TempDirectoryHelper;
import com.tc.util.Assert;
import com.tctest.runner.AbstractTransparentApp;

public final class LuceneDistributedConcurrentLoadTestApp extends AbstractTransparentApp {

  private static final String SEARCH_FIELD = "contents";
  private final CyclicBarrier barrier;

  public LuceneDistributedConcurrentLoadTestApp(String appId, ApplicationConfig cfg, ListenerProvider listenerProvider) {
    super(appId, cfg, listenerProvider);
    barrier = new CyclicBarrier(getParticipantCount());
  }

  public void run() {
    try {
      boolean writerNode = false;
      if (barrier.await() == 0) writerNode = true;
      LuceneSampleDataIndex index = null;
      if (writerNode) index = new LuceneSampleDataIndex(getTempDirectory(true));
      barrier.await();
      if (!writerNode) index = new LuceneSampleDataIndex(getTempDirectory(false));
      barrier.await();

      // RAMDirectory dir = index.publishDirectory();
      // String[] files = dir.list();
      // long size = 0;
      // for (int i = 0; i < files.length; i++) {
      // System.out.println(files[i]);// XXX
      // size += dir.fileLength(files[i]);
      // }
      // System.out.println("SIZE: " + size);

      int count = index.query("buddha").length();
      Assert.assertEquals(count, 0);
      barrier.await();
      if (writerNode) index.put(SEARCH_FIELD, "buddha");
      barrier.await();
      count = index.query("buddha").length();
      Assert.assertEquals(count, 1);
      count = index.query("lamb").length();
      Assert.assertEquals(count, 14);
      barrier.await();
      if (writerNode) index.put(SEARCH_FIELD, "Mary had a little lamb.");
      barrier.await();
      count = index.query("lamb").length();
      Assert.assertEquals(count, 15);

      // for (int i = 0; i < files.length; i++) {
      // System.out.println(files[i]);// XXX
      // size += dir.fileLength(files[i]);
      // }
      // System.out.println("SIZE: " + size);

    } catch (BrokenBarrierException e) {
      barrier.reset();
      notifyError(e);
    } catch (InterruptedException e) {
      barrier.reset();
      notifyError(e);
    } catch (Exception e) {
      barrier.reset();
      notifyError(e);
    }
  }

  public static void visitL1DSOConfig(ConfigVisitor visitor, DSOClientConfigHelper config) {
    config.addIncludePattern(CyclicBarrier.class.getName());
    config.addIncludePattern("org.apache.lucene.store..*");
    config.addIncludePattern(LuceneSampleDataIndex.class.getName());
    String className = LuceneDistributedConcurrentLoadTestApp.class.getName();
    config.addIncludePattern(className);
    config.addWriteAutolock("* " + CyclicBarrier.class.getName() + "*.*(..)");

    LockDefinition ramDirectoryWrite = new LockDefinition("ramDirectory", ConfigLockLevel.WRITE);
    ramDirectoryWrite.commit();
    // LockDefinition ramDirectoryRead = new LockDefinition("ramDirectory", ConfigLockLevel.READ);
    // ramDirectoryRead.commit();
    //    
    // config.addLock("* org.apache.lucene.store.RAMDirectory", null);
    // config.addLock("* org.apache.lucene.store.RAMDirectory", null);
    // config.addLock("* org.apache.lucene.store.RAMDirectory", null);
    // config.addLock("* org.apache.lucene.store.RAMDirectory", null);
    // config.addLock("* org.apache.lucene.store.RAMDirectory", null);
    // config.addLock("* org.apache.lucene.store.RAMDirectory", null);
    // config.addLock("* org.apache.lucene.store.RAMDirectory", null);
    // config.addLock("* org.apache.lucene.store.RAMDirectory", null);
    // config.addLock("* org.apache.lucene.store.RAMDirectory", null);
    // config.addLock("* org.apache.lucene.store.RAMDirectory", null);

    config.addLock("* org.apache.lucene.store.RAMOutputStream.flushBuffer(..)", ramDirectoryWrite);
    // config.addWriteAutolock("* " + LuceneSampleDataIndex.class.getName() + "*.*(..)");
    // config.addRoot("directory", LuceneSampleDataIndex.class.getName() + ".directory");

    // - - close()
    // w + createOutput(String name)
    // w deleteFile(String name)
    // r fileExists(String name)
    // r fileLength(String name)
    // r fileModified(String name)
    // r * list()
    // - - makeLock(String name)
    // r + openInput(String name)
    // w renameFile(String from, String to)
    // w touchFile(String name)
    // config.addWriteAutolock("* org.apache.lucene.store.RAMDirectory.*.*(..)");

    // config.addWriteAutolock("* org.apache.lucene.store.RAMDirectory*.createOutput(..)");
    // config.addWriteAutolock("* org.apache.lucene.store.RAMDirectory.deleteFile(..)");
    // config.addWriteAutolock("*org.apache.lucene.store.RAMDirectory.renameFile(..)");
    // config.addWriteAutolock("* org.apache.lucene.store.RAMDirectory.touchFile(..)");
    // config.addReadAutolock("* org.apache.lucene.store.RAMDirectory.fileExists(..)");
    // config.addReadAutolock("* org.apache.lucene.store.RAMDirectory.fileLength(..)");
    // config.addReadAutolock("* org.apache.lucene.store.RAMDirectory.fileModified(..)");
    // config.addReadAutolock("* org.apache.lucene.store.RAMDirectory.list()");
    // config.addReadAutolock("* org.apache.lucene.store.RAMDirectory.openInput(..)");
    // config.addWriteAutolock("* java.util.Hashtable$*.*(..)");
    // config.addWriteAutolock("* java.util.Hashtable*.*(..)");
    // config.addRoot("files", "org.apache.lucene.store.RAMDirectory.files");

    // XXX
    // addWriteAutolock("* java.util.Hashtable$*.*(..)");
    // addWriteAutolock("* java.util.Hashtable*.*(..)");
    // addWriteAutolock("* java.util.Vector*.*(..)");

    config.addRoot("directory", LuceneSampleDataIndex.class.getName() + ".directory");
    config.addRoot("barrier", className + ".barrier");
  }

  private File getTempDirectory(boolean clean) throws IOException {
    return new TempDirectoryHelper(getClass(), clean).getDirectory();
  }
}
